@@ -157,7 +157,14 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return add_device.json()
 
-    def add_model_device(self, device_name, serial_no, username="admin", password=""):
+    def add_model_device(self, name, serial_no, username="admin", password="", os_ver=6, mr=4, os_type="fos",
+                         platform=""):
+        # remove nonblocking from flags. With non-blocking the returned status looks like this even when the job failed, 
+        # since the creation status of the job is returned:
+        # [{'data': {'pid': 20172, 'taskid': 3194}, 'status': {'code': 0, 'message': 'OK'}, 'url': 'dvm/cmd/add/device'}]
+        #
+        # without nonblocking the failure reason is returned: 
+        # [{'status': {'code': -20010, 'message': 'Serial number already in use'}, 'url': 'dvm/cmd/add/device'}]
         session = self.login()
         payload = {
             "method": "exec",
@@ -167,18 +174,20 @@ class FortiManager:
                     "data": {
                         "adom": self.adom,
                         "flags": [
-                            "create_task",
-                            "nonblocking"
+                            "create_task"
                         ],
                         "device": {
-                            "name": device_name,
+                            "name": name,
                             "adm_usr": username,
                             "adm_pass": password,
                             "flags": 67371040,
                             "sn": serial_no,
-                            "platform_str": "FortiGate-VM64",
-                            "os_ver": 6,
-                            "mr": 2
+                            "platform_str": platform,
+                            "os_ver": os_ver,
+                            "mr": mr,
+                            "os_type": os_type,
+                            "mgmt_mode": "fmg",
+                            "device_action": "add_model",
                         }
                     }
                 }
@@ -310,6 +319,24 @@ class FortiManager:
             self.base_url, json=payload, verify=self.verify)
         return assign_meta.json()
 
+    def assign_meta_to_device_vdom(self, device, vdom, meta_name, meta_value):
+        """
+        Assign a meta tag to the device
+        :param device: name of the device
+        :param vdom: Specify the Vdom
+        :param meta_name: name of the meta tag
+        :param meta_value: value of the meta tag
+        :return: returns response from FortiManager API whether the request was successful or not.!
+        """
+        session = self.login()
+        payload = {"method": "update",
+                   "params": [{"url": f"/dvmdb/adom/root/device/{device}/vdom/{vdom}",
+                               "data": {"name": f"{device}", "meta fields": {f"{meta_name}": f"{meta_value}"}}}]}
+        payload.update({"session": self.sessionid})
+        assign_meta_vdom = session.post(
+            self.base_url, json=payload, verify=self.verify)
+        return assign_meta_vdom.json()
+
     # Firewall Object Methods
     def get_firewall_address_objects(self, name=False):
         """
@@ -590,6 +617,52 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return get_vip_objects.json()["result"]
 
+    # Firewall Interfaces
+    def get_device_interfaces(self, device):
+        """
+        Get interface details from the devices.
+        :param device: Specify name of the device.
+        """
+        session = self.login()
+        payload = \
+            {
+                "method": "exec",
+                "params": [
+                    {"url": "sys/proxy/json",
+                     "data": {"target": [f"adom/{self.adom}/device/{device}"], "action": "get",
+                              "resource": "/api/v2/monitor/system/interface/select?&global=1&include_vlan=1"}}]}
+        payload.update(session=self.sessionid)
+        get_interfaces = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_interfaces.json()["result"]
+
+    # Services
+    def get_services(self):
+        """
+                Get interface details from the devices.
+                :param device: Specify name of the device.
+                """
+        session = self.login()
+        payload = \
+            {"method": "get",
+             "params": [{"url": f"pm/config/adom/{self.adom}/obj/firewall/service/custom/Custom_Service_1"}]}
+
+        payload.update(session=self.sessionid)
+        services = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return services.json()["result"]
+
+    def get_service(self, name):
+        """
+        Get interface details from the devices.
+        :param name: Specify name of the device.
+        """
+        session = self.login()
+        payload = \
+            {"method": "get", "params": [{"url": f"pm/config/adom/{self.adom}/obj/firewall/service/custom/{name}"}]}
+
+        payload.update(session=self.sessionid)
+        service = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return service.json()["result"]
+
     # Firewall Policies Methods
     def get_firewall_policies(self, policy_package_name="default", policyid=False):
         """
@@ -617,7 +690,7 @@ class FortiManager:
 
     def add_firewall_policy(self, policy_package_name: str, name: str, source_interface: str,
                             source_address: str, destination_interface: str, destination_address: str,
-                            service: str, schedule="always", action=1, logtraffic=2):
+                            service: str, nat='disable', schedule="always", action=1, logtraffic=2):
         """
         Create your own policy in FortiManager using the instance parameters.
         :param policy_package_name: Enter the name of the policy package                eg. "default"
@@ -627,6 +700,7 @@ class FortiManager:
         :param destination_interface: Enter the source interface in a string format     eg. "port2"
         :param destination_address: Enter the dst. address object name                  eg. "WAN_100.25.1.63_32"
         :param service: Enter the service you want to permit or deny in string          eg. "ALL_TCP"
+        :param nat: Enter enable or disable for nat in a string format                  eg. 'disable'
         :param schedule: Schedule time is kept 'always' as default.
         :param action: Permit(1) or Deny(0) the traffic. Default is set to Permit.
         :param logtraffic: Specify if you need to log all traffic or specific in int format.
@@ -649,7 +723,8 @@ class FortiManager:
                         "service": service,
                         "srcaddr": source_address,
                         "srcintf": source_interface,
-                        "action": action
+                        "action": action,
+                        "nat": nat
                     },
                     "url": f"pm/config/adom/{self.adom}/pkg/{policy_package_name}/firewall/policy/"
                 }
